#ifndef _ViewCell_H__
#define _ViewCell_H__

#include "Intersectable.h"
#include "Containers.h"

class Mesh;

//namespace GtpVisibilityPreprocessor {
  
class BspInterior;
class BspPvs;
/**
	View cell represented as a mesh
*/
class ViewCell: public Intersectable
{
public:
	/** Default constructor taking a mesh which represents the shape of
		the viewcell.
	*/
	ViewCell(Mesh *mesh);

	/** Returns pointer to the mesh which represents the shape of the viewcell.
	*/
	Mesh *GetMesh();

	/** Returns pointer to PVS. 
		@returns PVS, i.e., the visible BSP tree nodes.
	*/
	BspPvs *GetPVS();

	AxisAlignedBox3 GetBox();
  
	int CastRay(Ray &ray);
  
	bool IsConvex();
	bool IsWatertight();
	float IntersectionComplexity();

	int Type() const;

	void GetRandomSurfacePoint(Vector3 &point, Vector3 &normal);

	/** Derives viewcells from object container.
		@param objects the intersectables the viewcells are derived from
		@param viewCells the viewcells are returned in this container
		@param maxViewCells if > 0, indicates the maximim number of viewcells that will be created
	*/
	static void DeriveViewCells(const ObjectContainer &objects, 
								ViewCellContainer &viewCells, 
								const int maxViewCells);

	static ViewCell *Convert2ViewCell(Face *face);
protected:

	/// the mesh defining the geometry of this viewcell
	Mesh *mMesh;
	/// the PVS (i.e., the visible bsp tree nodes)
	BspPvs *mPvs;

};
  
//}; // GtpVisibilityPreprocessor

#endif
